@@ -21,7 +21,7 @@
 #include <sys/types.h>
 #include <unistd.h>
 #ifdef USE_ECONF
-#include <libeconf.h>
+#include "pam_econf.h"
 #endif
 
 #include <security/pam_modules.h>
@@ -52,9 +52,9 @@ typedef struct var {
 #define DEFAULT_USER_ENVFILE    ".pam_environment"
 #define DEFAULT_USER_READ_ENVFILE 0
 
-#define DEFAULT_CONF_FILE	(SCONFIGDIR "/pam_env.conf")
-#ifdef VENDOR_SCONFIGDIR
-#define VENDOR_DEFAULT_CONF_FILE (VENDOR_SCONFIGDIR "/pam_env.conf")
+#define DEFAULT_CONF_FILE	(SCONFIG_DIR "/pam_env.conf")
+#ifdef VENDOR_SCONFIG_DIR
+#define VENDOR_DEFAULT_CONF_FILE (VENDOR_SCONFIG_DIR "/pam_env.conf")
 #endif
 
 #define GOOD_LINE    0
@@ -217,33 +217,32 @@ econf_read_file(const pam_handle_t *pamh, const char *filename, const char *deli
       char *vendor_dir = NULL, *sysconf_dir;
       if (subpath != NULL && subpath[0] != '\0') {
 #ifdef VENDORDIR
-	if (asprintf(&vendor_dir, "%s%s/%s/", base_dir, VENDORDIR, subpath) < 0) {
+	if ((vendor_dir = pam_asprintf("%s%s/%s/", base_dir, VENDORDIR, subpath)) == NULL) {
 	  pam_syslog(pamh, LOG_ERR, "Cannot allocate memory.");
 	  return PAM_BUF_ERR;
 	}
 #endif
-	if (asprintf(&sysconf_dir, "%s%s/%s/", base_dir, SYSCONFDIR, subpath) < 0) {
+	if ((sysconf_dir = pam_asprintf("%s%s/%s/", base_dir, SYSCONFDIR, subpath)) == NULL) {
 	  pam_syslog(pamh, LOG_ERR, "Cannot allocate memory.");
 	  free(vendor_dir);
 	  return PAM_BUF_ERR;
 	}
       } else {
 #ifdef VENDORDIR
-	if (asprintf(&vendor_dir, "%s%s/", base_dir, VENDORDIR) < 0) {
+	if ((vendor_dir = pam_asprintf("%s%s/", base_dir, VENDORDIR)) == NULL) {
 	  pam_syslog(pamh, LOG_ERR, "Cannot allocate memory.");
 	  return PAM_BUF_ERR;
 	}
 #endif
-	if (asprintf(&sysconf_dir, "%s%s/", base_dir, SYSCONFDIR) < 0) {
+	if ((sysconf_dir = pam_asprintf("%s%s/", base_dir, SYSCONFDIR)) == NULL) {
 	  pam_syslog(pamh, LOG_ERR, "Cannot allocate memory.");
 	  free(vendor_dir);
 	  return PAM_BUF_ERR;
 	}
       }
 
-      D(("Read configuration from directory %s and %s", vendor_dir, sysconf_dir));
-      error = econf_readDirs (&key_file, vendor_dir, sysconf_dir, name, suffix,
-			      delim, "#");
+      error = pam_econf_readconfig (&key_file, vendor_dir, sysconf_dir, name, suffix,
+				    delim, "#", NULL, NULL);
       free(vendor_dir);
       free(sysconf_dir);
       if (error != ECONF_SUCCESS) {
@@ -273,7 +272,7 @@ econf_read_file(const pam_handle_t *pamh, const char *filename, const char *deli
       return PAM_ABORT;
     }
 
-    *lines = malloc((key_number +1)* sizeof(char**));
+    *lines = calloc((key_number + 1), sizeof(char**));
     if (*lines == NULL) {
       pam_syslog(pamh, LOG_ERR, "Cannot allocate memory.");
       econf_free(keys);
@@ -281,19 +280,18 @@ econf_read_file(const pam_handle_t *pamh, const char *filename, const char *deli
       return PAM_BUF_ERR;
     }
 
-    (*lines)[key_number] = 0;
-
+    size_t n = 0;
     for (size_t i = 0; i < key_number; i++) {
       char *val;
 
       error = econf_getStringValue (key_file, NULL, keys[i], &val);
-      if (error != ECONF_SUCCESS) {
+      if (error != ECONF_SUCCESS || val == NULL) {
 	pam_syslog(pamh, LOG_ERR, "Unable to get string from key %s: %s",
 		   keys[i],
 		   econf_errString(error));
       } else {
         econf_unescnl(val);
-        if (asprintf(&(*lines)[i],"%s%c%s", keys[i], delim[0], val) < 0) {
+        if (((*lines)[n] = pam_asprintf("%s%c%s", keys[i], delim[0], val)) == NULL) {
 	  pam_syslog(pamh, LOG_ERR, "Cannot allocate memory.");
           econf_free(keys);
           econf_freeFile(key_file);
@@ -303,6 +301,7 @@ econf_read_file(const pam_handle_t *pamh, const char *filename, const char *deli
 	  return PAM_BUF_ERR;
 	}
 	free (val);
+	n++;
       }
     }
 
@@ -334,7 +333,7 @@ static int read_file(const pam_handle_t *pamh, const char*filename, char ***line
       (void) fclose(conf);
       return PAM_BUF_ERR;
     }
-    (*lines)[i] = 0;
+    (*lines)[i] = NULL;
     while (_pam_line_assemble(conf, &buffer, '\0') > 0) {
       char *p = buffer.assembled;
       char **tmp = NULL;
@@ -356,7 +355,7 @@ static int read_file(const pam_handle_t *pamh, const char*filename, char ***line
         _pam_line_buffer_clear(&buffer);
         return PAM_BUF_ERR;
       }
-      (*lines)[i] = 0;
+      (*lines)[i] = NULL;
     }
 
     (void) fclose(conf);
@@ -832,7 +831,7 @@ _define_var(pam_handle_t *pamh, int ctrl, VAR *var)
   int retval = PAM_SUCCESS;
 
   D(("Called."));
-  if (asprintf(&envvar, "%s=%s", var->name, var->value) < 0) {
+  if ((envvar = pam_asprintf("%s=%s", var->name, var->value)) == NULL) {
     pam_syslog(pamh, LOG_CRIT, "out of memory");
     return PAM_BUF_ERR;
   }
@@ -1074,7 +1073,7 @@ handle_env (pam_handle_t *pamh, int argc, const char **argv)
       pam_syslog(pamh, LOG_ERR, "No such user!?");
     }
     else {
-      if (asprintf(&envpath, "%s/%s", user_entry->pw_dir, user_env_file) < 0)
+      if ((envpath = pam_asprintf("%s/%s", user_entry->pw_dir, user_env_file)) == NULL)
 	{
 	  pam_syslog(pamh, LOG_CRIT, "Out of memory");
 	  return PAM_BUF_ERR;
