@@ -48,9 +48,9 @@
 #include "faillock_config.h"
 #include "faillock.h"
 
-#define FAILLOCK_DEFAULT_CONF SCONFIGDIR "/faillock.conf"
-#ifdef VENDOR_SCONFIGDIR
-#define VENDOR_FAILLOCK_DEFAULT_CONF VENDOR_SCONFIGDIR "/faillock.conf"
+#define FAILLOCK_DEFAULT_CONF SCONFIG_DIR "/faillock.conf"
+#ifdef VENDOR_SCONFIG_DIR
+#define VENDOR_FAILLOCK_DEFAULT_CONF VENDOR_SCONFIG_DIR "/faillock.conf"
 #endif
 
 static void PAM_FORMAT((printf, 3, 4)) PAM_NONNULL((3))
