@@ -34,9 +34,7 @@
  * OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#if defined(HAVE_CONFIG_H)
 #include <config.h>
-#endif
 
 #include <pwd.h>
 #include <shadow.h>
@@ -76,7 +74,7 @@
 #define RANDOM_DEVICE "/dev/urandom"
 #endif
 
-#define DEFAULT_OLD_PASSWORDS_FILE SCONFIGDIR "/opasswd"
+#define DEFAULT_OLD_PASSWORDS_FILE SCONFIG_DIR "/opasswd"
 
 typedef struct {
   char *user;
@@ -121,23 +119,32 @@ parse_entry (char *line, opwd *data)
   return 0;
 }
 
+/* Return 1 if the passwords are equal, 0 if they are not, and -1 on error. */
 static int
 compare_password(const char *newpass, const char *oldpass)
 {
   char *outval;
   int retval;
 #ifdef HAVE_CRYPT_R
-  struct crypt_data output;
+  struct crypt_data *cdata;
 
-  output.initialized = 0;
+  cdata = calloc(1, sizeof(*cdata));
+  if (!cdata)
+    return -1;
 
-  outval = crypt_r (newpass, oldpass, &output);
+  outval = crypt_r (newpass, oldpass, cdata);
 #else
   outval = crypt (newpass, oldpass);
 #endif
 
   retval = outval != NULL && strcmp(outval, oldpass) == 0;
+
+#ifdef HAVE_CRYPT_R
+  pam_overwrite_object(cdata);
+  free(cdata);
+#else
   pam_overwrite_string(outval);
+#endif
   return retval;
 }
 
@@ -202,13 +209,19 @@ check_old_pass, const char *user, const char *newpass, const char *filename, int
 
       do {
 	oldpass = strsep (&running, delimiters);
-	if (oldpass && strlen (oldpass) > 0 &&
-	    compare_password(newpass, oldpass) )
-	  {
-	    if (debug)
-	      pam_syslog (pamh, LOG_DEBUG, "New password already used");
-	    retval = PAM_AUTHTOK_ERR;
-	    break;
+	if (oldpass && strlen (oldpass) > 0) {
+	    int rc;
+
+	    rc = compare_password(newpass, oldpass);
+	    if (rc) {
+	      if (rc < 0)
+	        pam_syslog (pamh, LOG_ERR, "Cannot allocate crypt data");
+	      else if (debug)
+	        pam_syslog (pamh, LOG_DEBUG, "New password already used");
+
+	      retval = PAM_AUTHTOK_ERR;
+	      break;
+	    }
 	  }
       } while (oldpass != NULL);
     }
@@ -236,9 +249,9 @@ save_old_pass, const char *user, int howmany, const char *filename, int debug UN
   /* Define opasswd file and temp file for opasswd */
   const char *opasswd_file =
 	  (filename != NULL ? filename : DEFAULT_OLD_PASSWORDS_FILE);
-  char *opasswd_tmp;
+  char *opasswd_tmp = pam_asprintf("%s.tmpXXXXXX", opasswd_file);
 
-  if (asprintf (&opasswd_tmp, "%s.tmpXXXXXX", opasswd_file) < 0)
+  if (opasswd_tmp == NULL)
     return PAM_BUF_ERR;
 
   pwd = pam_modutil_getpwnam (pamh, user);
@@ -377,7 +390,7 @@ save_old_pass, const char *user, int howmany, const char *filename, int debug UN
 
 	    if (parse_entry (buf, &entry) == 0)
 	      {
-		char *out = NULL;
+		char *out;
 
 		found = 1;
 
@@ -414,30 +427,19 @@ save_old_pass, const char *user, int howmany, const char *filename, int debug UN
 		  }
 
 		if (entry.count == 1)
-		  {
-		    if (asprintf (&out, "%s:%s:%d:%s\n",
-				  entry.user, entry.uid, entry.count,
-				  oldpass) < 0)
-		      {
-		        free (save);
-			retval = PAM_AUTHTOK_ERR;
-			fclose (oldpf);
-			fclose (newpf);
-			goto error_opasswd;
-		      }
-		  }
+		  out = pam_asprintf("%s:%s:%d:%s\n",
+				     entry.user, entry.uid, entry.count, oldpass);
 		else
+		  out = pam_asprintf("%s:%s:%d:%s,%s\n",
+				     entry.user, entry.uid, entry.count,
+				     entry.old_passwords, oldpass);
+		if (out == NULL)
 		  {
-		    if (asprintf (&out, "%s:%s:%d:%s,%s\n",
-				  entry.user, entry.uid, entry.count,
-				  entry.old_passwords, oldpass) < 0)
-		      {
-		        free (save);
-			retval = PAM_AUTHTOK_ERR;
-			fclose (oldpf);
-			fclose (newpf);
-			goto error_opasswd;
-		      }
+		    free (save);
+		    retval = PAM_AUTHTOK_ERR;
+		    fclose (oldpf);
+		    fclose (newpf);
+		    goto error_opasswd;
 		  }
 
 		if (fputs (out, newpf) < 0)
@@ -471,7 +473,7 @@ save_old_pass, const char *user, int howmany, const char *filename, int debug UN
     {
       char *out;
 
-      if (asprintf (&out, "%s:%d:1:%s\n", user, pwd->pw_uid, oldpass) < 0)
+      if ((out = pam_asprintf("%s:%d:1:%s\n", user, pwd->pw_uid, oldpass)) == NULL)
 	{
 	  retval = PAM_AUTHTOK_ERR;
 	  if (oldpf)
@@ -519,8 +521,8 @@ save_old_pass, const char *user, int howmany, const char *filename, int debug UN
       goto error_opasswd;
     }
 
-  char *opasswd_backup;
-  if (asprintf (&opasswd_backup, "%s.old", opasswd_file) < 0)
+  char *opasswd_backup = pam_asprintf("%s.old", opasswd_file);
+  if (opasswd_backup == NULL)
     {
       retval = PAM_BUF_ERR;
       goto error_opasswd;
