@@ -61,8 +61,10 @@
 #include "hmacsha1.h"
 #endif /* WITH_OPENSSL */
 
-#ifdef USE_LOGIND
+#ifdef USE_LIBSYSTEMD
 #include <systemd/sd-login.h>
+#elif defined(USE_LIBELOGIND)
+#include <elogind/sd-login.h>
 #else
 #include <utmp.h>
 #endif
@@ -72,6 +74,7 @@
 #include <security/pam_ext.h>
 #include <security/pam_modutil.h>
 #include "pam_inline.h"
+#include "pam_i18n.h"
 
 /* The default timeout we use is 5 minutes, which matches the sudo default
  * for the timestamp_timeout parameter. */
@@ -185,11 +188,11 @@ format_timestamp_name(char *path, size_t len,
 		      const char *user)
 {
 	if (strcmp(ruser, user) == 0) {
-		return snprintf(path, len, "%s/%s/%s", timestamp_dir,
-				ruser, tty);
+		return pam_snprintf(path, len, "%s/%s/%s", timestamp_dir,
+				    ruser, tty);
 	} else {
-		return snprintf(path, len, "%s/%s/%s:%s", timestamp_dir,
-				ruser, tty, user);
+		return pam_snprintf(path, len, "%s/%s/%s:%s", timestamp_dir,
+				    ruser, tty, user);
 	}
 }
 
@@ -368,7 +371,7 @@ get_timestamp_name(pam_handle_t *pamh, int argc, const char **argv,
 	}
 	/* Generate the name of the file used to cache auth results.  These
 	 * paths should jive with sudo's per-tty naming scheme. */
-	if (format_timestamp_name(path, len, tdir, tty, ruser, user) >= (int)len) {
+	if (format_timestamp_name(path, len, tdir, tty, ruser, user) < 0) {
 		return PAM_AUTH_ERR;
 	}
 	if (debug) {
@@ -481,6 +484,7 @@ pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, const char **argv)
 
 #ifdef WITH_OPENSSL
 		if (hmac_size(pamh, debug, &maclen)) {
+			close(fd);
 			return PAM_AUTH_ERR;
 		}
 #else
@@ -846,7 +850,7 @@ main(int argc, char **argv)
 
 	/* Generate the name of the timestamp file. */
 	if (format_timestamp_name(path, sizeof(path), TIMESTAMPDIR,
-				  tty, user, target_user) >= (int) sizeof(path)) {
+				  tty, user, target_user) < 0) {
 		fprintf(stderr, "path too long\n");
 		return 4;
 	}
