@@ -67,6 +67,7 @@
 #include <security/pam_modutil.h>
 
 #include "pam_inline.h"
+#include "pam_i18n.h"
 #include "pam_cc_compat.h"
 #include "md5.h"
 #include "support.h"
@@ -278,7 +279,7 @@ static int _unix_run_update_binary(pam_handle_t *pamh, unsigned long long ctrl,
 	else
 		args[3] = "0";
 
-        snprintf(buffer, sizeof(buffer), "%d", remember);
+        pam_sprintf(buffer, "%d", remember);
         args[4] = buffer;
 
 	DIAG_PUSH_IGNORE_CAST_QUAL;
@@ -544,7 +545,7 @@ static int _pam_unix_approve_pass(pam_handle_t * pamh
 	const char *remark = NULL;
 	int retval = PAM_SUCCESS;
 
-	D(("&new=%p, &old=%p", pass_old, pass_new));
+	D(("&new=%p, &old=%p", pass_new, pass_old));
 	D(("new=[%s]", pass_new));
 	D(("old=[%s]", pass_old));
 
